require 'blueauth/version'
require 'blueauth/error'
require 'blueauth/certificates'
require 'net-ldap'

class Blueauth

  BPBASE = 'ou=bluepages,o=ibm.com'
  BGBASE = 'ou=memberlist,ou=ibmgroups,o=ibm.com'
  BPHOSTS = [['bluepages.ibm.com',636]]

  old_verbose, $VERBOSE = $VERBOSE, nil
  Net::LDAP::LDAPControls::PAGED_RESULTS = false
  $VERBOSE = old_verbose

  def initialize

    cert_store = OpenSSL::X509::Store.new
    cert_store.add_cert OpenSSL::X509::Certificate.new(NEW_CERT)
    cert_store.add_cert OpenSSL::X509::Certificate.new(OLD_CERT)

    @ldap = Net::LDAP.new hosts: BPHOSTS, connect_timeout: 15, encryption: {
      method: :simple_tls,
      tls_options: {
        ssl_version: :TLSv1_2,
        verify_mode: OpenSSL::SSL::VERIFY_PEER,
        cert_store: cert_store
      }
    }
  end

  # using this method a user can be authenticated
  # Intraned ID, password are mandatory
  def authenticate(id, password)
    user = search id.strip
    unless user.nil?
      @ldap.auth user[:dn], password.strip
      begin
        auth = @ldap.bind
      rescue => e
        raise BlueError, "BluePages Bind issue -> #{e.message}"
      end
      if auth
        groups = bluegroups user[:dn]
        return user.merge({groups: groups})
      else
        return nil
      end
    end
  end

  # Tries to find the given user id in Enterprise Directory and the result will be an LDAP object
  # user id can be
  #   - Intranet ID (must contain '@' sign)
  #   - Notes ID (must contain '/' sign)
  #   - Common name (none of the previous two)
  # return object contains
  # :name, :country, :intranetid, :dn
  def search(id)
    if id.include? '@'
      searchfield = 'mail'
    elsif id.include? '/'
      searchfield = 'notesid'
      email_parts = id.split('/')
      id = ''
      c = 1
      email_parts.each do |part|
        id =
          case c
            when 1
              'CN='+part
            when email_parts.count
              id + '/O='+part
            else
              id + '/OU='+part
          end
        c += 1
      end
    else
      searchfield = 'cn'
    end
    filter = Net::LDAP::Filter.eq(searchfield, id) & Net::LDAP::Filter.eq('objectclass', "ibmPerson")
    begin
      user_array = @ldap.search(base: BPBASE, filter: filter, size: 1)
    rescue => e
      raise BlueError, "BluePages Search issue -> #{e.message}"
    end

    if user_array.count == 0
      result = nil
    else
      user = user_array.first
      result = {name: user.cn.first, callupname: user&.callupname&.first || '', country: user.co.first, intranetid: user.preferredidentity.first, dn: user.dn}
    end
    return result
  end

  def bluegroups(dn)
    result = []
    filter = Net::LDAP::Filter.eq('uniquemember', dn)
    begin
      bgres = @ldap.search(base: BGBASE, filter: filter, attributes: ['cn'])
      bgres.each {|g| result << g.cn.first}
    rescue => e
      raise BlueError, "BlueGroup Search issue -> #{e.message}"
    end
    return result
  end

end
